#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "uifconfig.h"

extern int optind;

int main(int argc, char **argv) {
	char *devname = NULL;
	int dump = 0;
	int c;

#define OPTS "ahH?"

	while((c = getopt(argc, argv, OPTS)) != -1) {
	switch(c) {
	case 'a':
		dump = 1;
		break;

	case 'H':
	case 'h':
	case '?':
		fprintf(stderr, "??\n");
		exit(1);	
	}}

	if(dump) {
		char **l;
		int i;
		int c;

		c = uif_list(0, &l);
		if(c < 0) {
			fprintf(stderr, "ifconfig: %s: %s\n", 
				uif_strerror(uif_errno), strerror(errno));
			exit(1);
		} 

		if(c == 0) {
			fprintf(stderr, "ifconfig: no configured interfaces\n");
			exit(1);
		}

		for(i = 0; i < c; i++) {
			struct in_addr ia;
			u_int32_t flag;

			void *handle = uif_get(l[i], 0);
			if(!handle) {
				fprintf(stderr, "ifconfig: interface \"%s\": %s: %s\n",
					l[i], uif_strerror(uif_errno),
					strerror(errno));
				continue;		
			}

			printf("%s\t\tinet addr:", l[i]);

			ia.s_addr = uif_getip4addr(handle, NULL);
			
			printf("%s  Bcast:", inet_ntoa(ia));

			ia.s_addr = uif_getip4bcst(handle, NULL);

			printf("%s  Mask:", inet_ntoa(ia));

			ia.s_addr = uif_getip4mask(handle, NULL);

			printf("%s\n", inet_ntoa(ia));

			flag = uif_getflags(handle, NULL);			

			printf("\t\t%s\n\n", uif_flagstring(flag));						

			uif_release(handle);			
		}
		
		exit(0);
	}

	argc -= optind;
	argv += optind;

	devname = argv[0];

	if(!devname) {
		fprintf(stderr, "??\n");
		exit(1);
	}

	{
		void *handle = uif_get(devname, 0);
		if(!handle) {
			fprintf(stderr, "ifconfig: interface \"%s\": %s: %s\n",
					devname,
					uif_strerror(uif_errno),
					strerror(errno));
			exit(1);
		}
	
		argc -= 1;
		argv += 1;

#define UIFERR fprintf(stderr, "ifconfig: interface \"%s\": %s: %s\n", 	\
				devname, 				\
				uif_strerror(uif_errno),		\
				strerror(errno))

#define UIFARG fprintf(stderr, "ifconfig: invalid arguments\n")

		while(argc) {
			u_int32_t ip;

			if(!strcasecmp(argv[0], "inet")) {
				if(argc < 2) {
					UIFARG;
					exit(1);
				}

				ip = inet_addr(argv[1]);

				if(uif_setip4addr(handle, ip) < 0) {
					UIFERR;
					exit(1);
				}		

				argc -= 2;
				argv += 2;		
			} else if(!strcasecmp(argv[0], "broadcast")) {
				if(argc < 2) {
					UIFARG;
					exit(1);
				}

				ip = inet_addr(argv[1]);

				if(uif_setip4bcst(handle, ip) < 0) {
					UIFERR;
					exit(1);
				}		

				argc -= 2;
				argv += 2;		
			} else if(!strcasecmp(argv[0], "netmask")) {
				if(argc < 2) {
					UIFARG;
					exit(1);
				}

				ip = inet_addr(argv[1]);

				if(uif_setip4mask(handle, ip) < 0) {
					UIFERR;
					exit(1);
				}		

				argc -= 2;
				argv += 2;		
			} else if(!strcasecmp(argv[0], "destination")) {
				if(argc < 2) {
					UIFARG;
					exit(1);
				}

				ip = inet_addr(argv[1]);

				if(uif_setip4dst(handle, ip) < 0) {
					UIFERR;
					exit(1);
				}		

				argc -= 2;
				argv += 2;		
			} else if(!strcasecmp(argv[0], "+alias")) {
				if(argc < 2) {
					UIFARG;
					exit(1);
				}

				ip = inet_addr(argv[1]);

				if(uif_addip4alias(handle, ip) < 0) {
					UIFERR;
					exit(1);
				}		

				argc -= 2;
				argv += 2;		
			} else if(!strcasecmp(argv[0], "-alias")) {
				if(argc < 2) {
					UIFARG;
					exit(1);
				}

				ip = inet_addr(argv[1]);

				if(uif_delip4alias(handle, ip) < 0) {
					UIFERR;
					exit(1);
				}						

				argc -= 2;
				argv += 2;		
			} else if(!strcasecmp(argv[0], "up")) {
				u_int32_t f = uif_getflags(handle, NULL);
				
				f |= UIF_UP;

				if(uif_setflags(handle, f) < 0) {
					UIFERR;
					exit(1);
				}

				argc -= 1;
				argv += 1;
			} else if(!strcasecmp(argv[0], "loop")) {
				u_int32_t f = uif_getflags(handle, NULL);
				
				f |= UIF_LOOP;

				if(uif_setflags(handle, f) < 0) {
					UIFERR;
					exit(1);
				}

				argc -= 1;
				argv += 1;
			} else if(!strcasecmp(argv[0], "noarp")) {
				u_int32_t f = uif_getflags(handle, NULL);
				
				f &= ~UIF_ARP;

				if(uif_setflags(handle, f) < 0) {
					UIFERR;
					exit(1);
				}

				argc -= 1;
				argv += 1;
			} else if(!strcasecmp(argv[0], "promisc")) {
				u_int32_t f = uif_getflags(handle, NULL);
				
				f |= UIF_PROMISC;

				if(uif_setflags(handle, f) < 0) {
					UIFERR;
					exit(1);
				}

				argc -= 1;
				argv += 1;
			} else if(!strcasecmp(argv[0], "notrailers")) {
				u_int32_t f = uif_getflags(handle, NULL);
				
				f &= ~UIF_TRAILERS;

				if(uif_setflags(handle, f) < 0) {
					UIFERR;
					exit(1);
				}

				argc -= 1;
				argv += 1;
			} else if(!strcasecmp(argv[0], "allmulti")) {
				u_int32_t f = uif_getflags(handle, NULL);
				
				f |= UIF_ALLMULTI;

				if(uif_setflags(handle, f) < 0) {
					UIFERR;
					exit(1);
				}

				argc -= 1;
				argv += 1;
			} else if(!strcasecmp(argv[0], "anymulti")) {
				u_int32_t f = uif_getflags(handle, NULL);
				
				f |= UIF_ANYMULTI;

				if(uif_setflags(handle, f) < 0) {
					UIFERR;
					exit(1);
				}

				argc -= 1;
				argv += 1;
			} else if(!strcasecmp(argv[0], "dynamic")) {
				u_int32_t f = uif_getflags(handle, NULL);
				
				f |= UIF_DYNAMIC;

				if(uif_setflags(handle, f) < 0) {
					UIFERR;
					exit(1);
				}

				argc -= 1;
				argv += 1;
			} else if(!strcasecmp(argv[0], "down")) {
				u_int32_t f = uif_getflags(handle, NULL);
				
				f &= ~UIF_UP;

				if(uif_setflags(handle, f) < 0) {
					UIFERR;
					exit(1);
				}

				argc -= 1;
				argv += 1;
			} else {
				UIFARG;
				exit(1);
			}
		}
	}

	exit(0);
}

