#ifndef LINUX_H_SEEN
#define LINUX_H_SEEN

/* 
 * This include list brought to you by Linux net-tools ifconfig 1.34.
 * What, you want me to figure out what this include list is supposed
 * to be by myself? The hell with that!
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/sockio.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/if.h>
#include <net/if_arp.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>

#include <assert.h>

#include "uifconfig.h"

#include "common/xroutines.h"

/* Hack Hack Hack */

#ifndef IFF_DYNAMIC 
#define IFF_DYNAMIC 0x8000 
#endif

/*
 * Hello. I am the device-specific information stored in the "handle"
 * field of libifconfig public functions. I am not particularly 
 * efficient, but efficiency isn't an issue for ifconfig. Let me tell
 * you more about me:
 */

struct md_block {
	
	/* A socket file descriptor. Any socket file descriptor. 
 	 * Because Unix is arcane and silly, interface management
	 * is done via ioctl()s on sockets rather than on character
	 * devices in /dev.
 	 * 
 	 * "family" is provided by the library user, but right now
	 * the only meaningful value is "AF_INET".
	 */

	int fd;
	int family;

	/* As a convenience, we attempt to apply some semantic 
	 * meanings to libifconfig primitives, like setting the
	 * netmask and broadcast address sanely when setting the
	 * primary address. We need to be able to turn this off.
	 */
	int raw;

	struct ifreq ifr;

	/* Stored ifreq.ifr_flags from the last SIOCGIFFLAGS */

	int flags;

	/* Stored primary device address. */

	struct sockaddr_in pa;

	/* Stored device destination address */

	struct sockaddr_in da;

	/* Stored broadcast address */

	struct sockaddr_in ba;

	/* Stored netmask */

	struct sockaddr_in nm;

	/* Linux IP aliases are brutal, though not as bad as 4.4BSD's.
	 * Each alias is its own "sorta-netdevice", with an ifreq 
	 * associated with it. We store the whole list of ifreqs here
	 * rather than converting to something simpler. The ifreqs
	 * here use ifr_name and ifr_addr --- do not change them.
	 */

	int aliascnt;
	struct ifreq *aliases;
};

#endif
