#ifndef UIFCONFIG_H_SEEN
#define UIFCONFIG_H_SEEN

#ifdef SOLARIS
typedef uint64_t u_int64_t;
typedef uint32_t u_int32_t;
typedef uint16_t u_int16_t;
typedef uint8_t u_int8_t;

#define INADDR_NONE ((u_int32_t) -1) 

#endif

extern int uif_errno;

/* Retrieve a handle for the specified device and family, returning
 * NULL on failure.
 */

void *uif_get(char *dev, int family);

/* Retrieve the list of system interfaces. You are responsible
 * for freeing "names" and names[0..retval-1]
 */

int uif_list(int family, char ***names); 

/* XP ifconfig flags. These do NOT correspond to SIOCGIFCONF
 * flags (library maps them appropriately and may apply additional
 * flags to support semantic meanings.
 */

u_int32_t uif_getflags(void *handle, int *err);
int       uif_setflags(void *handle, u_int32_t f);

#define  	UIF_UP		0x1		/* UP/DOWN 		*/
#define 	UIF_LOOP	0x2		/* LOOPBACK/REAL 	*/
#define		UIF_ARP		0x4		/* ARP/NOARP 		*/
#define		UIF_PROMISC	0x8		/* PROMISC/NOT 		*/
#define		UIF_TRAILERS	0x16		/* ETHERTRAILERS/NO 	*/
#define		UIF_ALLMULTI	0x32		/* MC DISCRIMINATE/NO 	*/
#define 	UIF_ANYMULTI	0x64		/* MC ENABLED/NO 	*/
#define		UIF_DYNAMIC	0x128		/* TEMP ADDR/PERM ADDR 	*/

/* MD device flags. These correspond directly to SIOGCIFCONF or 
 * whatever your OS supports. There's intentionally no "retrieve"
 * operation for this.
 */

int uif_setmdflags(void *handle, u_int32_t mdf);

/* Set/get the primary interface address. */

u_int32_t uif_getip4addr(void *handle, int *err);
int	  uif_setip4addr(void *handle, u_int32_t ip);

/* Set/get the netmask. This defaults to 0xFFFFFF00 */

u_int32_t uif_getip4mask(void *handle, int *err);
int	  uif_setip4mask(void *handle, u_int32_t ip);

/* Set/get the broadcast address. This will default to 
 * the all-1's address based on the netmask. You will
 * almost never need to actually set this.
 */

u_int32_t uif_getip4bcst(void *handle, int *err);
int	  uif_setip4bcst(void *handle, u_int32_t ip);

/* Set/get point-to-point "destination" address */

u_int32_t uif_getip4dst(void *handle, int *err);
int	  uif_setip4dst(void *handle, u_int32_t ip);

/* Manage interface aliases. "count" can be -1 to 
 * signal error.
 */

u_int32_t *uif_getip4aliases(void *handle, int *count);
int        uif_addip4alias(void *handle, u_int32_t ip);
int        uif_delip4alias(void *handle, u_int32_t ip);

/* Apply additional semantics (netmask/bcast addr) to 
 * address sets? Defaults to 0 ("yes").
 */

void uif_setraw(void *handle, int v);
int  uif_getraw(void *handle);

void uif_release(void *handle);

char *uif_strerror(int);
char *uif_flagstring(u_int32_t);

#define UIFE_UNKNOWN 		0
#define UIFE_IFLIST		1
#define UIFE_SETADDR		2
#define UIFE_SETFLAGS		3
#define UIFE_SETDSTADDR		4
#define UIFE_SETNETMASK 	5
#define UIFE_SETBCAST		6
#define UIFE_GETADDR		7
#define UIFE_GETFLAGS		8
#define UIFE_GETDSTADDR		9
#define UIFE_GETNETMASK 	10
#define UIFE_GETBCAST		11
#define UIFE_OPEN		12
#define UIFE_MAX		12

#endif
